using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Web.UI;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Xml;
using System.Drawing;

using Telerik.WebControls;

using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a bar that runs at the top of the page containing the Quick Access
	/// buttons and titles.
	/// </summary>
	[
	Designer(typeof(ApplicationBarDesigner)),
	ToolboxData("<{0}:ApplicationBar runat=\"server\"></{0}:ApplicationBar>"),
	ParseChildren(true)
	]
	public class ApplicationBar : RibbonControl, IPostBackEventHandler
	{

		#region Fields

		private ITemplate _placeHolder = null;
		private ControlCollection _placeHolderControls = null;
		private string _customizeScript = string.Empty;
		private string _customizeToolTip = string.Empty;
		private bool _autoPostBack = false;
		private string _customizeMenuID = string.Empty;
		private string _applicationMenuID = string.Empty;
		private string _ribbonTabStripID = string.Empty;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the ApplicationBar class.
		/// </summary>
		public ApplicationBar()
		{
			// Shared State
			// 0 - ApplicationName
			// 1 - DocumentName
			// 2 - ApplicationMenuID
			// 3 - RibbonTabStripID
			// 4 - RibbonMultipPageViewID
			// 5 - QuickAccessCustomizeMenuID
			// 6 - MinimizeRibbon
			// 7 - RibbonOpen
			// 8 - OriginallySelectedTabIndex
			this.SharedState.Add(new SharedStateItem("ApplicationName", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("DocumentName", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("ApplicationMenuID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("RibbonTabStripID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("RibbonMultiPageViewID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("CustomizeMenuID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("MinimizeRibbon", typeof(bool), false));
			this.SharedState.Add(new SharedStateItem("RibbonOpen", typeof(bool), true));
			this.SharedState.Add(new SharedStateItem("OriginallySelectedTabIndex", typeof(int), -1));
		}

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets the name of the application. The title is displayed in the format
		/// ApplicationName - DocumentName.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the name of the application. The title is displayed in the format ApplicationName - DocumentName."),
		Category("Appearance")
		]
		public string ApplicationName
		{
			get
			{
				string name = Convert.ToString(this.SharedState["ApplicationName"].Value);

				return name;
			}
			set
			{
				this.SharedState["ApplicationName"].Value = value;
			}
		}


		/// <summary>
		/// Gets or sets the name of the document. The title is displayed in the format
		/// ApplicationName - DocumentName.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the name of the document. The title is displayed in the format ApplicationName - DocumentName."),
		Category("Appearance")
		]
		public string DocumentName
		{
			get
			{
				string name = Convert.ToString(this.SharedState["DocumentName"].Value);

				return name;
			}
			set
			{
				this.SharedState["DocumentName"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets the ID of the menu associated with the application button. 
		/// N.B. On the server this is the control's ID, on the client this is the control's ClientID. 
		/// Empty if not set.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the ID of the menu associated with the application button."),
		Category("Behavior")
		]
		public virtual string ApplicationMenuID
		{
			get
			{
				return _applicationMenuID;
			}
			set
			{
				// The client ID will be calculated when shared state is saved as the associated
				// control may not have been loaded yet
				_applicationMenuID = value;
			}
		}

		/// <summary>
		/// Gets or sets the ID of the tab strip associated with the ribbon. 
		/// N.B. On the server this is the control's ID, on the client this is the control's ClientID. 
		/// Empty if not set.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the ID of the tab strip associated with the ribbon."),
		Category("Behavior")
		]
		public virtual string RibbonTabStripID
		{
			get
			{
				return _ribbonTabStripID;
			}
			set
			{
				// The client ID will be calculated when shared state is saved as the associated
				// control may not have been loaded yet
				_ribbonTabStripID = value;
			}
		}

		/// <summary>
		/// Gets or sets the ID of the menu associated with customizing the quick access toolbar. 
		/// N.B. On the server this is the control's ID, on the client this is the control's ClientID. 
		/// Empty if not set.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the ID of the menu used for customizing the quick access toolbar."),
		Category("Behavior")
		]
		public virtual string CustomizeMenuID
		{
			get
			{
				return _customizeMenuID;
			}
			set
			{
				// The client ID will be calculated when shared state is saved as the associated
				// control may not have been loaded yet
				_customizeMenuID = value;

				SetCustomizeButtonState();
			}
		}

		/// <summary>
		/// Gets or sets a tool tip to be displayed over the customize button.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the tool tip to display for the customize access toolbar button."),
		Category("Behavior")
		]
		public string CustomizeToolTip
		{
			get
			{
				return _customizeToolTip;
			}
			set
			{
				_customizeToolTip = value;
			}
		}

		/// <summary>
		/// Gets or set the client code to execute when the customization button is clicked.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the client code to execute when the customization button is clicked."),
		Category("Behavior")
		]
		public virtual string OnClientCustomizeClick
		{
			get
			{
				return _customizeScript;
			}
			set
			{
				_customizeScript = value;
			}
		}

		/// <summary>
		/// Gets or sets whether the ribbon is minimized.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether the ribbon is minimized."),
		Category("Appearance")
		]
		public virtual bool MinimizeRibbon
		{
			get
			{
				bool minimize = Convert.ToBoolean(this.SharedState["MinimizeRibbon"].Value);

				return minimize;
			}
			set
			{
				bool changed = (value != this.MinimizeRibbon);
				
				this.SharedState["MinimizeRibbon"].Value = value;

				if (changed)
				{
					this.RibbonOpen = !value;

					RadTabStrip tabStrip =
							ControlFinder.FindUniqueControl(this.Page, this.RibbonTabStripID) as RadTabStrip;

					if (tabStrip != null)
					{
						if (value)
						{
							tabStrip.SelectedIndex = -1;
						}
						else if (this.OriginallySelectedTabIndex != -1)
						{
							tabStrip.SelectedIndex = this.OriginallySelectedTabIndex;
						}
					}
				}
			}
		}

		/// <summary>
		/// Gets or sets whether the ribbon is open when minimized. If not minimized this 
		/// will always be true and setting it will have no affect.
		/// </summary>
		[
		DefaultValue(false),
		Browsable(false)
		]
		public virtual bool RibbonOpen
		{
			get
			{
				bool open = Convert.ToBoolean(this.SharedState["RibbonOpen"].Value);

				return open;
			}
			set
			{
				if (!this.MinimizeRibbon)
				{
					value = true;
				}

				this.SharedState["RibbonOpen"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets the placeholder for quick access buttons.
		/// </summary>
		[
		DefaultValue(null),
		PersistenceMode(PersistenceMode.InnerProperty),
		Browsable(false),
		TemplateInstance(TemplateInstance.Single)
		]
		public ITemplate PlaceHolder
		{
			get
			{
				return _placeHolder;
			}
			set
			{
				_placeHolder = value;
			}
		}

		/// <summary>
		/// Gets or sets the collection of controls to be added to the place holder at run time.
		/// Design-time controls are added first then run-time.
		/// </summary>
		[
		DefaultValue(null),
		Browsable(false)
		]
		public ControlCollection PlaceHolderControls
		{
			get
			{
				if (_placeHolderControls == null)
				{
					_placeHolderControls = new ControlCollection(this);
				}

				return _placeHolderControls;
			}
		}

		/// <summary>
		/// Gets or sets whether clicking the customize button causes a post back.
		/// This must be explicitly set to cause a post back even if an event handler has 
		/// been defined.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether clicking the customize button causes a post back."),
		Category("Behavior")
		]
		public bool AutoPostBack
		{
			get
			{
				return _autoPostBack;
			}
			set
			{
				_autoPostBack = value;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Sets text - allows use by control and designer.
		/// </summary>
		public override void FinalizeRender()
		{
			base.FinalizeRender();

			SetText();
			SetRibbonBarState();
			SetCustomizeButtonState();

			if (!this.DesignMode)
			{
				RenderClientOnCustomizeClick();
			}
		}

		#endregion Methods

		#region Events

		/// <summary>
		/// Raises the CustomizeClick event.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected virtual void OnCustomizeClick(EventArgs e)
		{
			if (this.CustomizeClick != null)
			{
				CustomizeClick(this, e);
			}
		}

		/// <summary>
		/// Occurs when the customize button is clicked.
		/// </summary>
		public event EventHandler CustomizeClick;

		#endregion Events

		#region Implementation Properties

		/// <summary>
		/// Gets the control's tag type.
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		/// <summary>
		/// Gets or sets the index of the selected tab when the Ribbon was minimized.
		/// -1 if not selected.
		/// <para />
		/// This is for internal use only and should not be called outside of the constrainsts
		/// of the state storage mechanism.
		/// </summary>
		protected virtual int OriginallySelectedTabIndex
		{
			get
			{
				int index = Convert.ToInt32(this.SharedState["OriginallySelectedTabIndex"].Value);

				return index;
			}
			set
			{
				this.SharedState["OriginallySelectedTabIndex"].Value = value;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Ensure we add runtime controls so they are available for event handler re-attachment.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			if (!this.DesignMode)
			{
				AddRuntimeControls();
			}
		}

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("class", "applicationBar");

			HtmlGenericControl startDiv = new HtmlGenericControl("div");
			startDiv.EnableViewState = false;
			startDiv.Attributes.Add("class", "applicationBarStart");

			this.Controls.Add(startDiv);

			HtmlGenericControl quickAccessStartDiv = new HtmlGenericControl("div");
			quickAccessStartDiv.EnableViewState = false;
			quickAccessStartDiv.Attributes.Add("class", "quickAccessStart");

			this.Controls.Add(quickAccessStartDiv);
			
			HtmlGenericControl quickAccessPlaceHolderDiv = new HtmlGenericControl("div");
			quickAccessPlaceHolderDiv.EnableViewState = false;
			quickAccessPlaceHolderDiv.ID = "PlaceHolder";
			quickAccessPlaceHolderDiv.Attributes.Add("class", "quickAccessPlaceHolder");

			if (this.PlaceHolder != null)
			{
				this.PlaceHolder.InstantiateIn(quickAccessPlaceHolderDiv);
			}

			this.Controls.Add(quickAccessPlaceHolderDiv);

			HtmlGenericControl quickAccessEndDiv = new HtmlGenericControl("div");
			quickAccessEndDiv.EnableViewState = false;
			quickAccessEndDiv.Attributes.Add("class", "quickAccessEnd");

			this.Controls.Add(quickAccessEndDiv);

			HtmlGenericControl customizeDiv = new HtmlGenericControl("div");
			customizeDiv.EnableViewState = false;
			customizeDiv.ID = "Customize";
			customizeDiv.Attributes.Add("class", "quickAccessCustomizeButton");

			this.Controls.Add(customizeDiv);

			HtmlGenericControl textDiv = new HtmlGenericControl("div");
			textDiv.EnableViewState = false;
			textDiv.ID = "Text";
			textDiv.Attributes.Add("class", "applicationBarText");
			
			this.Controls.Add(textDiv);
		}

		/// <summary>
		/// Create client script to cause a post back or execute client side java script.
		/// </summary>
		/// <param name="host">
		/// The host control to attach the onclick event to.
		/// </param>
		protected virtual void RenderClientOnCustomizeClick()
		{
			string postBackReference = string.Empty;
			string clientCode = string.Empty;

			HtmlGenericControl customizeDiv = FindControl("Customize") as HtmlGenericControl;
			
			if (!string.IsNullOrEmpty(this.CustomizeMenuID))
			{
				if (this.AutoPostBack)
				{
					PostBackOptions options = new PostBackOptions(this);
					options.AutoPostBack = true;
					postBackReference = this.Page.ClientScript.GetPostBackEventReference(options);
				}

				if (!string.IsNullOrEmpty(this.OnClientCustomizeClick))
				{
					clientCode = this.OnClientCustomizeClick;
				}

				if ((!string.IsNullOrEmpty(clientCode)) || (!string.IsNullOrEmpty(postBackReference)))
				{
					string code = clientCode + postBackReference;

					// Embedded JavaScript
					customizeDiv.Attributes.Add("onclick", code);
				}
			}
		}

		/// <summary>
		/// Refreshes the application bar's shared state when the page is complete. This is 
		/// required because the application bar references other controls that are not rendered
		/// until after the bar. This means it cannot access them during construction.
		/// </summary>
		protected override void RegisterClientMemberScript(HtmlTextWriter writer)
		{
			base.RegisterClientMemberScript(writer);

			RibbonControlFinder finder = new RibbonControlFinder(this.Page);
			RadTabStrip tabStrip = finder.FindRibbonTabStrip();

			// Embedded JavaScript - Named dependency
			string script = this.ClientMemberName + ".attachTabStripEventHandler();";

			this.Page.ClientScript.RegisterStartupScript(this.GetType(), this.UniqueID, script, true);
		}

		/// <summary>
		/// Sets the originally selected tab prior to saving shared state.
		/// </summary>
		protected override void SaveSharedState()
		{
			if (this.Page != null)
			{
				// Calculate before saving state as the controls may not be available when the 
				// properties were set

				string clientId = GetClientID(this.Page, this.ApplicationMenuID);
				this.SharedState["ApplicationMenuID"].Value = clientId;

				clientId = GetClientID(this.Page, this.CustomizeMenuID);
				this.SharedState["CustomizeMenuID"].Value = clientId;

				clientId = GetClientID(this.Page, this.RibbonTabStripID);
				this.SharedState["RibbonTabStripID"].Value = clientId;

				RibbonControlFinder finder = new RibbonControlFinder(this.Page);
				RadTabStrip tabStrip = finder.FindRibbonTabStrip();

				if (tabStrip != null) 
				{
					// This is calculated from the tab strip and not stored in a property.
					// It is only used in the client scripts to show/hide all page views
					this.SharedState["RibbonMultiPageViewID"].Value = tabStrip.MultiPage.ClientID;

					if (tabStrip.SelectedIndex != -1)
					{
						this.OriginallySelectedTabIndex = tabStrip.SelectedTab.Index;
					}
				}
			}

			base.SaveSharedState();
		}

		/// <summary>
		/// Sets the title, in the format DocumentName - ApplicationName. If one or the other 
		/// is not specified the hyphen is not displayed
		/// </summary>
		protected virtual void SetText()
		{
			string title = string.Empty;

			if (!string.IsNullOrEmpty(this.DocumentName))
			{
				title += this.DocumentName;
			}

			if ((!string.IsNullOrEmpty(this.DocumentName)) && (!string.IsNullOrEmpty(this.ApplicationName)))
			{
				title += " - ";
			}

			if (!string.IsNullOrEmpty(this.ApplicationName))
			{
				title += this.ApplicationName;
			}

			HtmlGenericControl div = FindControl("Text") as HtmlGenericControl;

			if (div != null)
			{
				div.InnerText = title;
			}
		}

		/// <summary>
		/// Sets the state of the ribbon bar tabs/multipage.
		/// </summary>
		protected virtual void SetRibbonBarState()
		{
			if (this.Page != null)
			{
				RibbonControlFinder finder = new RibbonControlFinder(this.Page);
				RadTabStrip tabStrip = finder.FindRibbonTabStrip();

				if (tabStrip != null)
				{
					RadMultiPage multiPage = tabStrip.MultiPage;

					if (multiPage != null)
					{
						if (this.MinimizeRibbon)
						{
							if (this.RibbonOpen)
							{
								multiPage.CssClass = "ribbonBarFloat";
							}
							else
							{
								multiPage.CssClass = "ribbonBarClosed";
							}
						}
						else
						{
							multiPage.CssClass = "ribbonBar";
						}
					}
				}
			}
		}

		/// <summary>
		/// Shows or hides the customize button.
		/// </summary>
		protected virtual void SetCustomizeButtonState()
		{
			HtmlGenericControl customizeDiv = FindControl("Customize") as HtmlGenericControl;

			customizeDiv.Visible = (!string.IsNullOrEmpty(this.CustomizeMenuID));
			customizeDiv.Attributes.Add("title", this.CustomizeToolTip);
		}

		/// <summary>
		/// Adds controls added at run-time before rendering.
		/// </summary>
		private void AddRuntimeControls()
		{
			HtmlGenericControl placeHolderDiv = FindControl("PlaceHolder") as HtmlGenericControl;

			foreach (Control control in this.PlaceHolderControls)
			{
				placeHolderDiv.Controls.Add(control);
			}
		}

		/// <summary>
		/// Writes the object's details to the specified writer.
		/// </summary>
		/// <param name="writer">
		/// The writer to write to.
		/// </param>
		protected override void SerializeContent(XmlWriter writer)
		{
			base.SerializeContent(writer);

			writer.WriteAttributeString("ApplicationName", this.ApplicationName);
			writer.WriteAttributeString("DocumentName", this.DocumentName);
			writer.WriteAttributeString("MinimizeRibbon", this.MinimizeRibbon.ToString());
			writer.WriteAttributeString("RibbonOpen", this.RibbonOpen.ToString());
			writer.WriteAttributeString("OriginallySelectedTabIndex", this.OriginallySelectedTabIndex.ToString());

			RadTabStrip tabStrip = ControlFinder.FindUniqueControl(this.Page, this.RibbonTabStripID) as RadTabStrip;

			if (tabStrip != null)
			{
				writer.WriteAttributeString("SelectedTabIndex", tabStrip.SelectedIndex.ToString());
			}
		}

		/// <summary>
		/// Deserializes the object from the specified node.
		/// </summary>
		/// <param name="element">
		/// The element to get property values from.
		/// </param>
		public override void Deserialize(XmlElement element)
		{
			base.Deserialize(element);

			if (element.HasAttribute("ApplicationName"))
			{
				string applicationName = element.Attributes["ApplicationName"].Value;
				this.ApplicationName = applicationName;
			}

			if (element.HasAttribute("DocumentName"))
			{
				string documentName = element.Attributes["DocumentName"].Value;
				this.DocumentName = documentName;
			}

			if (element.HasAttribute("MinimizeRibbon"))
			{
				bool minimizeRibbon = Convert.ToBoolean(element.Attributes["MinimizeRibbon"].Value);
				this.MinimizeRibbon = minimizeRibbon;
			}

			if (element.HasAttribute("RibbonOpen"))
			{
				bool ribbonOpen = Convert.ToBoolean(element.Attributes["RibbonOpen"].Value);
				this.RibbonOpen = ribbonOpen;
			}

			if (element.HasAttribute("OriginallySelectedTabIndex"))
			{
				int originallySelectedTabIndex = Convert.ToInt32(element.Attributes["OriginallySelectedTabIndex"].Value);
				this.OriginallySelectedTabIndex = originallySelectedTabIndex;
			}

			if (element.HasAttribute("SelectedTabIndex"))
			{
				RadTabStrip tabStrip = ControlFinder.FindUniqueControl(this.Page, this.RibbonTabStripID) as RadTabStrip;

				if (tabStrip != null)
				{
					int index = Convert.ToInt32(element.Attributes["SelectedTabIndex"].Value);
					tabStrip.SelectedIndex = index;

					if (tabStrip.MultiPage != null)
					{
						tabStrip.MultiPage.SelectedIndex = index;
					}
				}
			}
		}

		#endregion Implementation Methods

		#region IPostBackEventHandler Members

		/// <summary>
		/// Raises the click event;
		/// </summary>
		/// <param name="eventArgument">
		/// The event data.
		/// </param>
		public void RaisePostBackEvent(string eventArgument)
		{
			OnCustomizeClick(EventArgs.Empty);
		}

		#endregion IPostBackEventHandler Members

	}

}
