using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.Design.WebControls;
using System.Web.UI.WebControls;


namespace RussellMason.Web.UI.Office2007.Design
{

	/// <summary>
	/// Basic designer for a table containing 3 rows.
	/// </summary>
	public class ThreeRowLayoutDesigner : CompositeControlDesigner
	{

		#region Fields

		private ThreeRowLayout _control; 

		#endregion Fields

		#region Contructors

		/// <summary>
		/// Initializes a new instance of the ThreeRowLayoutDesigner class.
		/// </summary>
		public ThreeRowLayoutDesigner()
		{
		} 

		#endregion Contructors

		#region Properties

		/// <summary>
		/// Gets whether the control can be sized at design-time.
		/// </summary>
		public override bool AllowResize
		{
			get
			{
				return true;
			}
		} 

		#endregion Properties

		#region Methods

		/// <summary>
		/// Initializes the control.
		/// </summary>
		/// <param name="component">
		/// The component to initialize.
		/// </param>
		public override void Initialize(IComponent component)
		{
			base.Initialize(component);

			_control = (ThreeRowLayout) component;
		}

		/// <summary>
		/// Generates design time HTML.
		/// </summary>
		/// <param name="regions">
		/// Template regions.
		/// </param>
		/// <returns>
		/// The design time HTML.
		/// </returns>
		public override String GetDesignTimeHtml(DesignerRegionCollection regions)
		{
			EditableDesignerRegion topRegion = new EditableDesignerRegion(this, "A", false);
			EditableDesignerRegion centreRegion = new EditableDesignerRegion(this, "B", false);
			EditableDesignerRegion bottomRegion = new EditableDesignerRegion(this, "C", false);

			regions.Add(topRegion);
			regions.Add(centreRegion);
			regions.Add(bottomRegion);

			return base.GetDesignTimeHtml();
		}

		/// <summary>
		/// Gets template content for editable regions.
		/// </summary>
		/// <param name="region">
		/// The region being edited.
		/// </param>
		/// <returns>
		/// The region content.
		/// </returns>
		public override string GetEditableDesignerRegionContent(EditableDesignerRegion region)
		{
			string content = string.Empty;
			IDesignerHost host = (IDesignerHost) this.Component.Site.GetService(typeof(IDesignerHost));

			if (host != null)
			{
				ITemplate template = null;

				if (region.Name == "A")
				{
					template = _control.TopPlaceHolder;
				}
				else if (region.Name == "B")
				{
					template = _control.CenterPlaceHolder;
				}
				else if (region.Name == "C")
				{
					template = _control.BottomPlaceHolder;
				}

				if (template != null)
				{
					content = ControlPersister.PersistTemplate(template, host);
				}
			}

			return content;
		}

		/// <summary>
		/// Sets template content for editable regions.
		/// </summary>
		/// <param name="region">
		/// The region being edited.
		/// </param>
		/// <param name="content">
		/// The content of the region.
		/// </param>
		public override void SetEditableDesignerRegionContent(EditableDesignerRegion region, string content)
		{
			if (content != null)
			{
				IDesignerHost host = (IDesignerHost) Component.Site.GetService(typeof(IDesignerHost));

				if (host != null)
				{
					ITemplate template = ControlParser.ParseTemplate(host, content);

					if (template != null)
					{
						if (region.Name.EndsWith("A"))
						{
							_control.TopPlaceHolder = template;
						}
						else if (region.Name.EndsWith("B"))
						{
							_control.CenterPlaceHolder = template;
						}
						else if (region.Name.EndsWith("C"))
						{
							_control.BottomPlaceHolder = template;
						}
					}
				}
			}
		}

		#endregion Methods

		#region Implementation Methods

		/// <summary>
		/// Sets design-time regions against the table cells.
		/// </summary>
		protected override void CreateChildControls()
		{
			base.CreateChildControls();

			Table table = (Table) _control.Controls[0];

			if (table != null)
			{
				table.Rows[0].Cells[0].Attributes[DesignerRegion.DesignerRegionAttributeName] = "0";
				table.Rows[1].Cells[0].Attributes[DesignerRegion.DesignerRegionAttributeName] = "1";
				table.Rows[2].Cells[0].Attributes[DesignerRegion.DesignerRegionAttributeName] = "2";
			}
		} 

		#endregion Implementation Methods

	}

}
