using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;

using Telerik.WebControls;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Base master page for containing Ribbon controls.
	/// </summary>
	public class RibbonMasterPage : MasterWebPage
	{

		#region Fields

		private RadAjaxManager _ajaxManager = null;
		private RibbonControlFinder _finder = null;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the RibbonMasterPage page.
		/// </summary>
		public RibbonMasterPage()
		{
		}

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets a manager for supplying AJAX services.
		/// </summary>
		[
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public RadAjaxManager AjaxManager
		{
			get
			{
				if (_ajaxManager == null)
				{
					_ajaxManager = new RadAjaxManager();
					_ajaxManager.ID = "RibbonMasterPageAjaxManager";

					this.Page.Form.Controls.Add(_ajaxManager);
				}

				return _ajaxManager;
			}
		}

		/// <summary>
		/// Gets a finder to help locate strongly type instances of ribbon items.
		/// </summary>
		public RibbonControlFinder Finder
		{
			get
			{
				if (_finder == null)
				{
					_finder = new RibbonControlFinder(this);
				}

				return _finder;
			}
		} 

		#endregion Properties

		#region Implementation Methods

		/// <summary>
		/// Calls additional configuration methods when the page has completed loading.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			ConfigureRibbonControls();
			RegisterAjaxSettings();
		}

		/// <summary>
		/// Allows additional Ribbon controls to be added or configured.
		/// </summary>
		protected virtual void ConfigureRibbonControls()
		{
		}

		/// <summary>
		/// Allows AJAX based controls to be associated with the manager via the AjaxManager 
		/// property.
		/// </summary>
		protected virtual void RegisterAjaxSettings()
		{
		}

		#endregion Implementation Methods

	}

}
